// \EXAMPLES\EX1311.CPP

// Example of STACKS using an abstract base class
//--------------------------------------------------------
files in this example
EX1311.CPP     all classes and main() defined in this file
//--------------------------------------------------------

#include <iostream.h>

class StackItem;           // forward declaration

//----------------------------------------------------
// CLASS Stack : a abstract base class for stacks that store
//               elements of type StackItem
//----------------------------------------------------

class Stack {
public:
   virtual void push(StackItem&) =0;
   virtual StackItem& pop() =0;
   virtual int empty() =0;
   virtual int full() =0;
   virtual void dumpStack() =0;
};

//----------------------------------------------------
// CLASS StackItem : A class for elements that are stored
//                   in Stack classes.
//----------------------------------------------------

class StackItem {
private:
   int value;
public:
   StackItem(int i = 0): value(i) { }
   int get() { return value; }
};

StackItem dummyItem;       // dummy StackItem to return on error

//----------------------------------------------------
// CLASS AStack : A class derived from Stack that
//                implements a stack as an array.
//----------------------------------------------------

class AStack: public Stack {
   int free;                // number of free elements in array
   int size;                // size of the array
   StackItem* pArray;       // pointer to the array
public:
   AStack( int capacity = 10);
   ~AStack();
        void push(StackItem&);
        StackItem& pop();
   int empty();
   int full();
   void dumpStack();
};

//----------------------------------------------------
// Definitions of inline functions for AStack
//----------------------------------------------------

void AStack::dumpStack() {
   int used = size - free;
        for (int i = 0; i < used; i++ ) {
                cout << pop().get() << endl;
      }
   }


AStack::AStack(int capacity) {
   pArray = new StackItem[capacity]; // allocate memory for array
   size = free = capacity;           //initialize size and free
}

AStack::~AStack() {
   delete[] pArray;                  // deallocate the array
}

void AStack::push(StackItem& s ) {
   if ( full() )
      cout << " Array full " << endl;
   else {
      pArray[size - free] = s;
      free--;
   }
}

StackItem& AStack::pop() {
   if ( empty() ) {
                cout << " Array empty " << endl;
                return dummyItem;
        }
        else {
        // return element on top of stack, remove it from stack
                return pArray[(size - free++) -1];
        }
}

int AStack::empty() {
        if ( free == size )       // if array empty, return true
                return 1;
   else
                return 0;              // otherwise return false
}

int AStack::full() {
   if ( free == 0 )          // if array is full, return true
      return 1;
   else
      return 0;              // otherwise, return false
}

//----------------------------------------------------
// CLASS LLStack : A class derived from Stack that
//                 implements a stack as a linked list.
//----------------------------------------------------

class LLStack: public Stack {
   struct Node                // structure for elements the list
   {
      StackItem datum;        // value of current element
      Node* next;             // pointer to next element
   };
   Node* pList;               // pointer to linked list
   int used;                  // number of elements now in list
public:
        LLStack( int size = 10);
   ~LLStack();
        void push(StackItem&);
   StackItem& pop();
   int empty();
   int full();
   void dumpStack();
};

//----------------------------------------------------
// Definitions of inline functions for AStack
//----------------------------------------------------



LLStack::LLStack( int ) {
        pList = NULL;
        used = 0;
}

LLStack::~LLStack() {
   Node* pTempNode = pList;
   while ( pTempNode != NULL ) {      // delete entire list
      pList = pList->next;
      delete pTempNode;
   }
}

void LLStack::push(StackItem& s) {
   Node* pNewNode = new Node;
   if ( pNewNode == NULL )
      cout << " Can't allocate stack " << endl;
   else {
      pNewNode->datum = s.get();      // assign data value
      pNewNode->next = pList;         // point to current list
      pList = pNewNode;               // --> to new node
      used++;
   }
}

StackItem& LLStack::pop() {
        Node* pTempNode = pList;
        if ( empty() ) {
                cout << " attempt to pop empty stack " << endl;
                return dummyItem;
        }
        else {
                StackItem& TempItem = pList->datum;
                pList = pList->next;
                delete pTempNode;       // delete node
                return TempItem;        // return data from node
        }
}

int LLStack::empty() {
   return ( pList == NULL );   // return true if list is empty
}

int LLStack::full() {
   return(0);                  // return false - list never full
}

void LLStack::dumpStack() {
   while ( pList != NULL ) {
      cout << pop().get() << endl;
   }
}

//----------------------------------------------------
// Simple client code that exercises the stack classes
//----------------------------------------------------

void useStack(Stack& stack) {
   for (int i=0; i < 10; i++) {
      StackItem s(i);        // define a StackItem
      stack.push(s);         // put StackItem on stack
   }
   stack.dumpStack();        // print and empty stack
   StackItem t(99);          // define a single StackItem
        stack.push(t);       // push it onto stack
                          // now pop item off stack and print it
   cout << "Here is top of stack: "
        << stack.pop().get() << endl;
}


//----------------------------------------------------
// Simple main() function to exercise AStack and LLStack
//----------------------------------------------------

void main() {
   AStack astack(10);        // define an Astack object
        LLStack llstack;
   cout << "Call client code with array stack " << endl;
   useStack(astack);
   cout << "Call client code with linked list stack " << endl;
   useStack(llstack);

   }
