// \EXAMPLES\EX17010B.H      - Stack.H

//----------------------------------------------------------
// Example of template container classes implenenting stacks.
// This version includes templates but no exception handling.
//----------------------------------------------------------
//----------------------------------------------------------
//  Exception handling is supported only by
//                      the IBM C++ Set/2 compiler
//  Templates are not supported by
//                      the Microcoft Visual C++ compiler
//  Two versions of this program are included:
//                      EX1701I.EXE - for IBM CSet II
//                      EX1701B.EXE - for Borland Turbo C++
//----------------------------------------------------------
// Files in this example:
// EX17010B.H       this file -- Stack.H       base class stack
// %F,15,EX17011B.H%EX17011B.H       AStack.H      derived array stack
// %F,15,EX17011B.CPP%EX17011B.CPP     AStack.CPP
// %F,15,EX17012B.H%EX17012B.H       LLStack.H     derived linked list stack
// %F,15,EX17012B.CPP%EX17012B.CPP     LLStack.CPP
// %F,15,EX17013B.H%EX17013B.H       Exception.H
// %F,15,EX17014B.H%EX17014B.H       Iterator.H       iterator class
// %F,15,EX1701B.CPP%EX1701B.CPP      main() without exception handling
// %F,15,EX1701.CPP%EX1701.CPP       main() with exception handling
//----------------------------------------------------------
#ifndef INCL_EX17010_H
#define INCL_EX17010_H

#include "EX17013B.H"

//---------------------------------------------------------
// TEMPLATE: class Stack: to create a base class for a stack
//                        which stores elements of type T
//
// COMMENTS: Notice all functions are pure virtuals
//---------------------------------------------------------
template<class T> class Stack
{
public:

   //------------------------------------------------------
   // FUNCTION: Push pushes an element of type T onto the
   //                stack
   // PARAMETERS: T y is an element of type T
   // RETURN VALUE: none
   // EXCEPTIONS: aborts if full
   //------------------------------------------------------
   virtual void
   Push( const T y) = 0;

   //------------------------------------------------------
   // FUNCTION: Pop pops an element of type T from the stack
   // PARAMETERS: none
   // RETURN VALUE: pointer to top element of the stack
   // EXCEPTIONS: aborts
   //------------------------------------------------------
   virtual T
   Pop() = 0;

   //------------------------------------------------------
   // FUNCTION: Full returns true if stack is full, false
   //                otherwise
   // PARAMETERS: none
   // RETURN VALUE: int
   //------------------------------------------------------
   virtual int
   Full() const = 0;

   //------------------------------------------------------
   // FUNCTION: Empty returns true if stack is empty, false
   //                 otherwise
   // PARAMETERS: none
   //
   // RETURN VALUE: int
   //------------------------------------------------------
   virtual int
   Empty() const = 0;

   //------------------------------------------------------
   // FUNCTION: OnTop returns the top element of stack
   // PARAMETERS: none
   // RETURN VALUE: T pointer to tope element of stack
   // EXCEPTIONS: aborts if empty
   //------------------------------------------------------
   virtual T
   OnTop() const = 0;

   //------------------------------------------------------
   // FUNCTION: NumElem returns number of elements in stack
   // PARAMETERS: none
   // RETURN VALUE: long the number of elements in the stack
   //------------------------------------------------------
   virtual long
   NumElem() const = 0;

};

#endif
