// \EXAMPLES\EX17012.CPP    - LLStack.CPP

//----------------------------------------------------------
// Example of template container classes implenenting stacks.
// This version includes exception handling and templates.
//----------------------------------------------------------
//----------------------------------------------------------
//  Exception handling is supported only by
//                      the IBM C++ Set/2 compiler
//  Templates are not supported by
//                      the Microcoft Visual C++ compiler
//  Two versions of this program are included:
//                      EX1701I.EXE - for IBM CSet II
//                      EX1701B.EXE - for Borland Turbo C++
//----------------------------------------------------------
// Files in this example:
// %F,15,EX17010.H%EX17010.H       Stack.H       base class stack
// %F,15,EX17011.H%EX17011.H       AStack.H      derived array stack
// %F,15,EX17011.CPP%EX17011.CPP     AStack.CPP
// %F,15,EX17012.H%EX17012.H       LLStack.H     derived linked list stack
// EX17012.CPP     this file -- LLStack.CPP
// %F,15,EX17013.H%EX17013.H       Exception.H
// %F,15,EX17014.H%EX17014.H       Iterator.H       iterator class
// %F,15,EX1701.CPP%EX1701.CPP      main() with exception handling
// %F,15,EX1701B.CPP%EX1701B.CPP     main() without exception handling
//----------------------------------------------------------
#ifndef INCL_EX17012_CPP
#define INCL_EX17012_CPP

#include <stdlib.h>
#include "EX17010.H"
#include "EX17012.H"

template<class T>
LLStack<T>::LLStack()
   : pList( (Node *)NULL), used( 0)
{
}


template<class T>
LLStack<T>::~LLStack()
{
   Node* pTempNode = (Node *)NULL;  // points to node in stack

   //stop when pList is null
   while( (pTempNode = pList) != (Node *)NULL)
   {
      pList = pList->next;        // remove node from stack
      delete pTempNode;           // deallocate node
   }
   pTempNode = (Node *)NULL;
}


template<class T> void
LLStack<T>::Push( const T y)
{
   Node* pNewNode = (Node *)NULL;   // points to new node

   pNewNode = new Node;             // allocate new node
   if (pNewNode == (Node *)NULL)
   {
      throw ( MemAlloc());      // throw exception - memory error
   }

   pNewNode->datum = y;        // assign data to new node

   // add node to stack
   pNewNode->next = pList;
   pList = pNewNode;

   pNewNode = (Node *)NULL;
   used++;
}


template<class T> T
LLStack<T>::Pop()
{
   Node* pTempNode = (Node *)NULL;  // pointer to top node

   if ( Empty())
   {
      throw( EmptyExcept());  // throw exception if stack empty
   }

   T datum( pList->datum);  // get the data from the top node

   // remove top node from stack
   pTempNode = pList;
   pList = pList->next;
   delete pTempNode;
   pTempNode = (Node *)NULL;

   used--;                // decrement the number used
   return datum;          // return data
}



template<class T> T
LLStack<T>::OnTop() const
{
   if (Empty())
   {
      throw( EmptyExcept());   // throw exception if stack  empty
   }
   return pList->datum;    // return data
}

#endif
